<?php # $Id: serendipity_plugin_api.php,v 1.8 2003/12/05 11:27:54 garvinhicking Exp $

if (IN_serendipity !== true) {
    die ('Don\'t hack!');
}

include_once 'serendipity_functions.inc.php';

/* This file defines the plugin API for serendipity.
 * By extending these classes, you can add your own code
 * to appear in the sidebar(s) of serendipity.
 *
 *
 * The system defines a number of built-in plugins; these are
 * identified by @class_name.
 *
 * Third-party plugins are identified by the name of the folder into
 * which they were uploaded (so there is no @ sign at the start of
 * their class name.
 *
 * The user creates instances of plugins; an instance is assigned
 * an identifier like this:
 *   classname:uniqid()
 *
 * The user can configure instances of plugins.
 */

class serendipity_plugin_api {

    function register_default_plugins()
    {
        $classes = serendipity_plugin_api::enum_plugin_classes();
        foreach ($classes as $class_name) {
            if ($class_name{0} == '@') {
                serendipity_plugin_api::create_plugin_instance($class_name);
            }
        }
    }

    /* Create an instance of a plugin.
     * $plugin_class_id is of the form:
     *    @class_name        for a built-in plugin
     * or
     *    plugin_dir_name    for a third-party plugin
     * returns the instance identifier for the newly created plugin.
     *
     * TO BE IMPLEMENTED:
     * If $copy_from_instance is not null, and identifies another plugin
     * of the same class, then the persistent state will be copied.
     * This allows the user to clone a plugin.
     */
    function create_plugin_instance($plugin_class_id, $copy_from_instance = null)
    {
        global $serendipity;

        $id = md5(uniqid(''));

        $key = $plugin_class_id . ':' . $id;

        $rs = serendipity_db_query("SELECT MAX(sort_order) as sort_order_max FROM {$serendipity['dbPrefix']}plugins", true, 'num');

        if (is_array($rs)) {
            $nextidx = intval($rs[0]+1);
        } else {
            $nextidx = 0;
        }

        serendipity_db_query("INSERT INTO {$serendipity['dbPrefix']}plugins (name, sort_order) values ('$key', $nextidx)");

        return $key;
    }

    function remove_plugin_instance($plugin_instance_id)
    {
        global $serendipity;
        serendipity_db_query("DELETE FROM {$serendipity['dbPrefix']}plugins where name='$plugin_instance_id'");
        serendipity_db_query("DELETE FROM {$serendipity['dbPrefix']}config  where name LIKE '$plugin_instance_id/%'");
    }

    /* Retrieves a list of available plugins */
    function enum_plugin_classes()
    {
        global $serendipity;

        $classes = array();

        /* built-in classes first */
        $cls = get_declared_classes();
        foreach ($cls as $class_name) {
            if (strncmp($class_name, 'serendipity_', 6)) {
                continue;
            }

            $p = get_parent_class($class_name);
            while ($p != 'serendipity_plugin' && $p !== false) {
                $p = get_parent_class($p);
            }

            if ($p == 'serendipity_plugin') {
                $classes[] = '@' . $class_name;
            }
        }

        /* third-party classes next */
        $ppath = (isset($serendipity['serendipityPath']) ? $serendipity['serendipityPath'] : '') . 'plugins';
        $d = opendir($ppath);
        while (($f = readdir($d)) !== false) {
            if ($f{0} == '.' || $f == 'CVS') {
                continue;
            }
            
            if (file_exists($ppath . '/' . $f . '/' . $f . '.php')) {
                $classes[] = $f;
            }
        }
        return $classes;
    }

    /* Retrieves a list of plugin instances */
    function enum_plugins($filter = '*')
    {
        global $serendipity;

        $sql = "SELECT * from {$serendipity['dbPrefix']}plugins ";

        if ($filter !== '*') {
            $sql .= "WHERE placement='$filter' ";
        }

        $sql .= ' ORDER BY placement, sort_order';

        return serendipity_db_query($sql);
    }

    /* Creates an instance of a named plugin */
    function &load_plugin($instance_id)
    {
        global $serendipity;

        list($name, $uniq) = explode(':', $instance_id);

        if ($name{0} == '@') {
            $class_name = substr($name, 1);
        } else {
            /* plugin from the plugins/ dir */
            if (!class_exists($name)) {

                include $serendipity['serendipityPath'] . 'plugins/' . $name . '/' . $name . '.php';

                if (!class_exists($name)) {
                    return false;
                }
            }

            $class_name = $name;
        }

        $p =& new $class_name($instance_id);

        return $p;
    }

    function update_plugin_placement($name, $placement, $order=null)
    {
        global $serendipity;

        $sql = "UPDATE {$serendipity['dbPrefix']}plugins set placement='$placement' ";

        if ($order !== null) {
            $sql .= ", sort_order=$order ";
        }

        $sql .= "WHERE name='$name'";

        return serendipity_db_query($sql);
    }

    function generate_plugins($side, $tagname = 'div')
    {
        $plugins = serendipity_plugin_api::enum_plugins($side);

        if (!is_array($plugins)) {
            return;
        }

        echo "<$tagname id=\"serendipity" . ucfirst($side) . "SideBar\">\n";
        foreach ($plugins as $plugin_data) {
            $plugin =& serendipity_plugin_api::load_plugin($plugin_data['name']);
            $title  = get_class($plugin);

            ob_start();
            $plugin->generate_content($title);
            $content = ob_get_contents();
            ob_end_clean();

            echo "    <div class=\"$plugin->wrap_class\">\n";
            echo "        <h3 class=\"$plugin->title_class\">$title</h3>\n";
            echo "        <div class=\"$plugin->content_class\">\n";
            echo "            $content\n";
            echo "        </div>\n";
            echo "    </div>\n";
        }

        echo "</$tagname>\n";
    }
}

/* holds a bunch of properties; you can have multiple
* properties with the same name */
class serendipity_property_bag {
    var $properties = array();
    var $name       = null;

    function add($name, $value)
    {
        $this->properties[] = array(
                                'name'  => $name,
                                'value' => $value
        );
    }

    function get($name)
    {
        $v = array();
        foreach ($this->properties as $data) {
            if ($data['name'] == $name) {
                $v[] = $data['value'];
            }
        }

        if (count($v) == 1) {
            return $v[0];
        }

        return implode(', ', $v);
    }

    function is_set($name)
    {
        foreach ($this->properties as $data) {
            if ($data['name'] == $name) {
                return true;
            }
        }

        return false;
    }
}

class serendipity_plugin {
    var $instance      = null;
    var $wrap_class    = 'serendipitySideBarItem';
    var $title_class   = 'serendipitySideBarTitle';
    var $content_class = 'serendipitySideBarContent';

    /* Be sure to call this method from your derived classes constructors,
     * otherwise your config data will not be stored or retrieved correctly
     */

    function serendipity_plugin($instance)
    {
        $this->instance = $instance;
    }

    /* Called by serendipity when it wants to display information
     * about your plugin.
     * You need to override this method in your child class.
     */
    function introspect(&$propbag)
    {
        $propbag->add('copyright', 'MIT License');
        $propbag->add('name'     , get_class($this));

        // $propbag->add(
        //   'configuration',
        //   array(
        //     'text field',
        //     'foo bar'
        //   )
        // );
        return true;
    }

    /* Called by serendipity when it wants to display the configuration
     * editor for your plugin.
     * $name is the name of a configuration item you added in
     * your instrospect method.
     * You need to fill the property bag with appropriate items
     * that describe the type and value(s) for that particular
     * configuration option.
     * You need to override this method in your child class if
     * you have configuration options.
     */
    function introspect_config_item($name, &$propbag)
    {
        return false;
    }

    /* Called by serendipity when it wants your plugin to display itself.
     * You need to set $title to be whatever text you want want to
     * appear in the item caption space.
     * Simply echo/print your content to the output; serendipity will
     * capture it and make things work.
     * You need to override this method in your child class.
     */
    function generate_content(&$title)
    {
        $title = 'Sample!';
        echo     'This is a sample!';
    }

    /* Fetches a configuration value for this plugin */
    function get_config($name, $defaultvalue = null)
    {
        $name = $this->instance . '/' . $name;

        return serendipity_get_config_var($name, $defaultvalue);
    }

    function set_config($name, $value)
    {
        $name = $this->instance . '/' . $name;

        return serendipity_set_config_var($name, $value);
    }
}

include_once 'serendipity_sidebar_items.php';

/* vim: set sts=4 ts=4 expandtab : */
?>
